# Speech_Recognition.py
#
# Copyright 2019 Ryota Higashi
# Copyright 2021 Eiichi Inohira
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Component

"""Speech_Recognition
"""

import sys
import queue
import time
from datetime import datetime
import threading
import xmlrpc.client

from pyrois import RoIS_Common, RoIS_HRI, IF_server

class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, c):
        self._component = c

    def start(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def set_parameter(self, recognizable_languages, languages, grammer, rule):
        arg = [recognizable_languages, languages, grammer, rule]
        self.Recognizable_Languages = recognizable_languages
        self.Languages = languages
        self.Grammer = grammer
        self.Rule = rule

        th = threading.Thread(target=self.execute, daemon=True)
        th.start()
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, c):
        self._component = c

    def component_status(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        with self._lock:
            c_status = self.c_status
        return (status, c_status)
    
    def get_parameter(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.c_status = RoIS_Common.Component_Status.READY.value
        return (status, self.Languages, self.Grammer, self.Rule)
    

class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, c):
        self._component = c
        self.event_queue = queue.Queue()

    def poll_event(self):
        """poll_event
        """
        msg = self.event_queue.get()
        return msg

    def speech_recognized(self, timestamp, recognized_text):
        """speech_recognition
        """
        msg = xmlrpc.client.dumps((timestamp, recognized_text), 'speech_recognized')
        self.event_queue.put(msg)
    
    def speech_input_started(self, timestamp):
        msg = xmlrpc.client.dumps((timestamp), 'speech_input_started')
        self.event_queue.put(msg)
    
    def speech_input_finished(self, timestamp):
        msg = xmlrpc.client.dumps((timestamp), 'speech_input_finished')
        self.event_queue.put(msg)


class Speech_Recognition(Event, Command, Query):
    """Speech_Recognition
    """
    def __init__(self, c, lock=None):
        super().__init__(c)
        self._component = c
        if lock is None:
            self._lock = threading.Lock()
        else:
            self._lock = lock
        
        self.c_status = RoIS_Common.Component_Status.READY.value
        self.parameter_queue = queue.Queue()
        
        self.Recognizable_Languages = set("")
        self.Languages = set("japanese")
        self.Grammer = "" #???
        self.Rule = "" #???
    
    def execute(self):  # dummy
        """execute component
        """
        arg = self.parameter_queue.get()
        with self._lock:
            self.c_status = RoIS_Common.Component_Status.BUSY.value

        time.sleep(1)

        with self._lock:
            self.c_status = RoIS_Common.Component_Status.READY.value


class component:
    """component
    """
    def __init__(self):
        self._state = False


def event_dispatch(sr):
    """event_dispatch
    """
    # sr.speech_recognized(datetime.now().isoformat(), [""])
    # time.sleep(0.1)
    # sr.speech_recognized(datetime.now().isoformat(), [""])


def example_sr(port):
    """example_sr
    """
    c = component()
    sr = Speech_Recognition(c)

    # start the timer to dispatch events
    # t = threading.Timer(0.1, event_dispatch, args=(sr,))
    # t.start()

    # start the XML-RPC server
    IF_server.IF_server(port).run(sd)

if __name__ == '__main__':
    example_sr(8009)


# Face_Detection.py
#
# Copyright 2019 Ryota Higashi
# Copyright 2021 Eiichi Inohira
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Component

"""Face_Detection
"""

import sys
import queue
import time
from datetime import datetime
import threading
import xmlrpc.client

from pyrois import RoIS_Common, RoIS_HRI, IF_server

class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, c):
        self._component = c

    def start(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, c):
        self._component = c

    def component_status(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        c_status = self.c_status
        return (status, c_status)


class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, c):
        self._component = c
        self.event_queue = queue.Queue()

    def poll_event(self):
        """poll_event
        """
        msg = self.event_queue.get()
        return msg

    def face_detected(self, timestamp, number):
        """face_detected
        """
        msg = xmlrpc.client.dumps((timestamp, number), 'face_detected')
        self.event_queue.put(msg)


class Face_Detection(Event, Command, Query):
    """Face_Detection
    """
    def __init__(self,c):
        super().__init__(c)
        self.c_status = RoIS_Common.Component_Status.READY.value


class component:
    """component
    """
    def __init__(self):
        self._state = False


def event_dispatch(fd):
    """event_dispatch
    """
    time.sleep(18)
    fd.face_detected(datetime.now().isoformat(), 1)
    time.sleep(0.1)
    fd.face_detected(datetime.now().isoformat(), 2)


def example_fd(port):
    """example_fd
    """
    c = component()
    fd = Face_Detection(c)

    # start the timer to dispatch events for unittest
    t = threading.Timer(0.1, event_dispatch, args=(fd,))
    t.start()

    # start the XML-RPC server
    IF_server.IF_server(port).run(fd)

if __name__ == '__main__':
    example_fd(8005)



# Person_Localization_Client.py
#
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Engine

"""Person_Localization_Client
"""

import threading
# import logging
import xmlrpc.client

from pyrois import RoIS_Common, RoIS_HRI


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def start(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.start())
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.stop())
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.suspend())
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.resume())
        return status

    def set_parameter(self, detection_threshold, minimum_interval):
        s = self._proxy.set_parameter(detection_threshold, minimum_interval)
        status = RoIS_HRI.ReturnCode_t(s)
        return status
    

class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def component_status(self):
        (status, c_status) = self._proxy.component_status()
        return (RoIS_HRI.ReturnCode_t(status), RoIS_Common.Component_Status(c_status))
    
    def get_parameter(self):
        (status, detection_threshold, minimum_interval) = self._proxy.get_parameter()
        return (RoIS_HRI.ReturnCode_t(status), detection_threshold, minimum_interval)


class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, uri):
        self._uri = uri
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []
        # if logger is not None:
        #     self.logger = logger
                
    def start_th(self):
        self.th = threading.Thread(target=self.event_loop, daemon=True)
        self.th.start()

    def event_loop(self):
        """event_loop
        """
        while True:
            try:
                self.poll_event()
            except ConnectionRefusedError:
                break

    def poll_event(self):
        """poll_event
        """
        msg = self._e_proxy.poll_event()
        (params, methodname) = xmlrpc.client.loads(msg)
        #self.logger.debug('poll_event: '+methodname)
        # print(params,methodname)
        if methodname == 'person_localized':
            self.person_localized(params[0], params[1], params[2])

    def person_localized(self, timestamp, person_ref, position_data):
        self.events.append((timestamp,person_ref,position_data))
        # print("person_localized",timestamp, person_ref, position_data)
        if self.q is not None:
            self.q.put(("person_localized",timestamp, person_ref, position_data))
        # self.logger.debug('received completed event'
        #                 + command_id
        #                 + RoIS_Service.Completed_Status(status).name)


class Person_Localization_Client(Command, Query, Event):
    """Person_localization_Client
    """
    def __init__(self, uri, q=None):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []
        self.q = q
        self.start_th()

# if __name__ == '__main__':
#     test = Person_Localizaion_Client('http://127.0.0.1:8003')
#     print("get:" + str(test.get_parameter()))
#     print("set:" + test.set_parameter('2000','5000').name)
#     print("get:" + str(test.get_parameter()))
#     print("set:" + test.set_parameter('1000','20').name)
#     print("get:" + str(test.get_parameter()))





# Gesture_Recognition.py
#
# Copyright 2019 Ryota Higashi
# Copyright 2021 Eiichi Inohira
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Component

"""Gesture_Recognition
"""

import sys
import queue
import time
from datetime import datetime
import threading
import xmlrpc.client

from pyrois import RoIS_Common, RoIS_HRI, IF_server


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, c):
        self._component = c

    def start(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, c):
        self._component = c

    def component_status(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        with self._lock:
            c_status = self.c_status
        return (status, c_status)
    
    def get_parameter(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return (status, self.Recognizable_Gestures)
    

class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, c):
        self._component = c
        self.event_queue = queue.Queue()

    def poll_event(self):
        """poll_event
        """
        msg = self.event_queue.get()
        return msg

    def gesture_recognized(self, timestamp, gesture_ref):
        """gesture_recognition
        """
        msg = xmlrpc.client.dumps((timestamp, gesture_ref), 'gesture_recognized')
        self.event_queue.put(msg)


class Gesture_Recognition(Event, Command, Query):
    """Gesture_Recognition
    """
    def __init__(self, c, lock=None):
        super().__init__(c)
        self._component = c
        if lock is None:
            self._lock = threading.Lock()
        else:
            self._lock = lock
        self.c_status = RoIS_Common.Component_Status.READY.value
        self.Recognizable_Gestures = set("")


class component:
    """component
    """
    def __init__(self):
        self._state = False


def event_dispatch(gr):
    """event_dispatch
    """
    # gr.gesture_recognized(datetime.now().isoformat(), [""])
    # time.sleep(0.1)
    # gr.gesture_recognized(datetime.now().isoformat(), [""])


def example_gr(port):
    """example_gr
    """
    c = component()
    gr = Gesture_Recognition(c, lock=None)

    # start the timer to dispatch events
    t = threading.Timer(0.1, event_dispatch, args=(gr,))
    t.start()

    # start the XML-RPC server
    IF_server.IF_server(port).run(gr)

if __name__ == '__main__':
    example_gr(8010)